using System;
using System.IO;
#if SILVERLIGHT
using System.Security.Cryptography;
#endif

namespace SharpCifs.Util.Sharpen
{
    public abstract class MessageDigest
	{
	    public void Digest (byte[] buffer, int o, int len)
		{
			byte[] d = Digest ();
			d.CopyTo (buffer, o);
		}

		public byte[] Digest (byte[] buffer)
		{
			Update (buffer);
			return Digest ();
		}

		public abstract byte[] Digest ();
		public abstract int GetDigestLength ();
		public static MessageDigest GetInstance (string algorithm)
		{
			switch (algorithm.ToLower ()) {
			//case "sha-1":
			//	return new MessageDigest<SHA1Managed> ();
			case "md5":
				return new MessageDigest<Md5Managed> ();
			}
			throw new NotSupportedException (string.Format ("The requested algorithm \"{0}\" is not supported.", algorithm));
		}

		public abstract void Reset ();
		public abstract void Update (byte[] b);
		public abstract void Update (byte b);
		public abstract void Update (byte[] b, int offset, int len);
	}


	public class MessageDigest<TAlgorithm> : MessageDigest where TAlgorithm : HashAlgorithm, new()
	{
		private TAlgorithm _hash;

		public MessageDigest ()
		{
			Init ();
		}

		public override byte[] Digest ()
		{
			byte[] hash = _hash.HashFinal();
			Reset ();
			return hash;
		}

		public void Dispose ()
		{
		}

		public override int GetDigestLength ()
		{
			return (_hash.HashSize() / 8);
		}

		private void Init ()
		{
			_hash = Activator.CreateInstance<TAlgorithm> ();
		}

		public override void Reset ()
		{
			Dispose ();
			Init ();
		}

		public override void Update (byte[] input)
		{
            _hash.HashCore(input, 0, input.Length);
		}

        byte[] bytebuf = new byte[1];

		public override void Update (byte input)
		{
            bytebuf[0] = input;
            _hash.HashCore(bytebuf, 0, 1);
        }

		public override void Update (byte[] input, int index, int count)
		{
			//if (count < 0)
			//	Console.WriteLine ("Argh!");
            _hash.HashCore(input, 0, input.Length);
		}
	}
}
