using System;
using System.IO;
using System.Threading.Tasks;

namespace SharpCifs.Util.Sharpen
{
    public class OutputStream : IDisposable
	{
		protected Stream Wrapped;

		public static implicit operator OutputStream (Stream s)
		{
			return Wrap (s);
		}

		public static implicit operator Stream (OutputStream s)
		{
			return s.GetWrappedStream ();
		}

        public virtual async Task Close()
		{
			if (Wrapped != null) {
				await Wrapped.FlushAsync ();
                Wrapped.Dispose();
			}
		}

        public void Dispose()
		{
			Close ();           
		}

		public virtual async Task FlushAsync ()
		{
			if (Wrapped != null) {
				await Wrapped.FlushAsync ();
			}
		}

		internal Stream GetWrappedStream ()
		{
			// Always create a wrapper stream (not directly Wrapped) since the subclass
			// may be overriding methods that need to be called when used through the Stream class
			return new WrappedSystemStream (this);
		}

		static internal OutputStream Wrap (Stream s)
		{
			OutputStream stream = new OutputStream ();
			stream.Wrapped = s;
			return stream;
		}

		public virtual async Task WriteAsync (int b)
		{
			if (Wrapped is WrappedSystemStream)
                await ((WrappedSystemStream)Wrapped).OutputStream.WriteAsync(b);
			else {
				if (Wrapped == null)
					throw new NotImplementedException ();
				Wrapped.WriteByte ((byte)b);
			}
		}

		public virtual async Task WriteAsync (byte[] b)
		{
			await WriteAsync (b, 0, b.Length);
		}

		public virtual async Task WriteAsync (byte[] b, int offset, int len)
		{
			if (Wrapped is WrappedSystemStream)
				await ((WrappedSystemStream)Wrapped).OutputStream.WriteAsync (b, offset, len);
			else {
				if (Wrapped != null) {
					Wrapped.Write (b, offset, len);
				} else {
					for (int i = 0; i < len; i++) {
						await WriteAsync (b[i + offset]);
					}
				}
			}
		}
	}
}
