﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using SharpCifs.Smb;
using Windows.Networking.Connectivity;
using Windows.Networking;

namespace TestSmb
{
    public sealed partial class MainPage : Page
    {
        public MainPage()
        {
            this.InitializeComponent();

            this.NavigationCacheMode = NavigationCacheMode.Required;
            SharpCifs.Config.SetProperty("jcifs.smb.client.useExtendedSecurity", "true");
            SharpCifs.Config.SetProperty("jcifs.smb.lmCompatibility", "3");

            SharpCifs.Config.SetProperty("jcifs.netbios.cachePolicy", "180");  // cache timeout: cache names
            SharpCifs.Config.SetProperty("jcifs.netbios.hostname", "windowsphone");
            SharpCifs.Config.SetProperty("jcifs.netbios.retryCount", "3");
            SharpCifs.Config.SetProperty("jcifs.netbios.retryTimeout", "5000");  // Name query timeout
            SharpCifs.Config.SetProperty("jcifs.smb.client.responseTimeout", "10000"); // increased for NAS where HDD is off!

            string laddr = GetLocalIPAddress();
            SharpCifs.Config.SetProperty("jcifs.netbios.laddr", laddr);
            SharpCifs.Config.SetProperty("jcifs.netbios.baddr", "255.255.255.255");
            SharpCifs.Config.SetProperty("jcifs.resolveOrder", "LMHOSTS,BCAST,DNS");
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
        }

        private SmbFile smb;

        private async void Button_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                String url = urlEdit.Text;  //  "smb://server/share/";
                if (!url.EndsWith("/"))
                    url += "/";   // it MUST end with a slash!
                textout.Text = "Trying: " + url;
                SharpCifs.Smb.NtlmPasswordAuthentication pass = new NtlmPasswordAuthentication(null, userEdit.Text, passEdit.Password);
                smb = new SmbFile(url, pass);
                SmbFile[] ls = await smb.ListFilesAsync();
                if (ls != null)
                {
                    int cnt = ls.Length;
                    string info = "Found " + cnt + " files!";
                    textout.Text = info;
                    // Read from first text file!
                    for (int i = 0; i < cnt; i++)
                    {
                        if (await ls[i].IsFile() && ls[i].GetName().ToLower().EndsWith(".txt"))
                        {
                            textout.Text = info;
                            SharpCifs.Util.Sharpen.InputStream stm = await ls[i].GetInputStream();
                            byte[] buf = new byte[65536];
                            int ret = 0;
                            Boolean first = true;
                            do
                            {
                                ret = await stm.ReadAsync(buf, 0, buf.Length);
                                if (ret > 0 && first)
                                {
                                    info += "\n" + ls[i].GetName();
                                    info += "\n" + System.Text.Encoding.UTF8.GetString(buf, 0, Math.Min(256, ret));
                                    first = false;
                                    textout.Text = info;
                                    break;
                                }
                            } while (ret > 0);
                            await stm.Close();
                            if (!(ret < 0 && first))
                                break;
                        }
                    }
                    // Try to create new text file
                    Boolean exists = false;
                    String name = "smb_test.txt";
                    for (int i = 0; i < cnt; i++)
                    {
                        if (ls[i].GetName() == name)
                        {
                            exists = true;
                            break;
                        }
                    }
                    if (!exists)
                    {
                        SmbFile smb2 = new SmbFile(url + name, pass);
                        await smb2.CreateNewFile();
                        SharpCifs.Util.Sharpen.OutputStream stm2 = await smb2.GetOutputStream();
                        String s = "This is a test.";
                        await stm2.WriteAsync(System.Text.Encoding.UTF8.GetBytes(s));
                        await stm2.FlushAsync();
                        await stm2.Close();
                    }
                }
            }
            catch (SmbException e2)
            {
                Exception e3 = e2.GetRootCause();
                String err = e2.Message;
                if (e3 != null && e2 != e3)
                    err += "\n" + e3.Message;
                textout.Text = err;
            }
            catch (Exception e3)
            {
                textout.Text = e3.Message;
            }
        }

        public static string GetLocalIPAddress()
        {
            var icp = NetworkInformation.GetInternetConnectionProfile();
            if (icp != null && icp.NetworkAdapter != null)
            {
                var hostnames = NetworkInformation.GetHostNames();
                if (hostnames != null)
                {
                    foreach (HostName name in hostnames)
                    {
                        if (name.IPInformation != null && name.IPInformation.NetworkAdapter != null
                            && name.Type == HostNameType.Ipv4
                            && name.IPInformation.NetworkAdapter.NetworkAdapterId == icp.NetworkAdapter.NetworkAdapterId)
                        {
                            return name.CanonicalName;
                        }
                    }
                }
            }
            return "";
        }

    }
}
